﻿using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Text.Json;
using System.Text.Json.Serialization;
using System.Windows;
using Corsinvest.ProxmoxVE.Api;
using Corsinvest.ProxmoxVE.Api.Extension;
using Corsinvest.ProxmoxVE.Api.Shared.Models.Cluster;
using Corsinvest.ProxmoxVE.Api.Shared.Models.Vm;
using MahApps.Metro.Controls.Dialogs;
using Proxmox.SpiceLauncher.Models;

namespace Proxmox.SpiceLauncher;

/// <summary>
/// Interaction logic for MainWindow.xaml
/// </summary>
public partial class MainWindow
{
    private PveClient? _client;
    private List<Settings> _settings;
    private List<VmWrapper> _vms = new();
    private readonly string _settingsFile;

    public MainWindow()
    {
        _settingsFile = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "ProxmoxSpiceLauncher", "settings.json");
        InitializeComponent();
    }
    private async void MetroWindow_Initialized(object sender, EventArgs e)
    {
        List<Settings>? settings = null;
        if (File.Exists(_settingsFile))
        {
            await using var f = File.OpenRead(_settingsFile);
            settings = await JsonSerializer.DeserializeAsync<List<Settings>>(f);
        }
        if (settings == null || settings.Count == 0)
        {
            settings =
            [
                new Settings()
                {
                    Server = "192.168.0.190"
                }
            ];
        }
        _settings = settings;
        ComboProfiles.ItemsSource = _settings;
        ComboProfiles.SelectedIndex = 0;
        ComboVms.ItemsSource = _vms;
    }

    private void ButtonSettings_Click(object sender, RoutedEventArgs e)
    {
        /*
        var settings = new SettingsWindow
        {
            Owner = this
        };
        settings.ShowDialog();
        */
    }

    private async Task DoEvent(Func<Task> e)
    {
        try
        {
            await e();
        }
        catch (Exception ex)
        {
            await this.ShowMessageAsync("Error", ex.Message);
        }
    }

    private async Task DoConnect()
    {
        _client = null;
        _vms.Clear();
        ButtonSpice.IsEnabled = false;
        if (ComboProfiles.SelectedItem is not Settings settings)
        {
            return;
        }

        _client = new PveClient(settings.Server);
        try
        {
            if (string.IsNullOrWhiteSpace(settings.Username) || string.IsNullOrWhiteSpace(settings.Password))
            {
                var login = await this.ShowLoginAsync("Login", $"Login to {settings.Server}", new LoginDialogSettings
                {
                    InitialUsername = settings.Username,
                    InitialPassword = settings.Password
                });
                settings.Username = login.Username;
                settings.Password = login.Password;
            }

            if (!await _client.LoginAsync(settings.Username, settings.Password))
            {
                settings.Password = "";
                throw new Exception("Login failed");
            }

            var vms = await _client.GetVmsAsync();
            foreach (var vm in vms)
            {
                if (!vm.IsRunning)
                {
                    continue;
                }

                if (vm.VmType != VmType.Qemu)
                {
                    continue;
                }

                var config = await _client.GetVmConfigAsync(vm.Node, vm.VmType, vm.VmId) as VmConfigQemu;

                if (config?.Vga != null && config.Vga.StartsWith("qxl"))
                {
                    _vms.Add(new VmWrapper(vm));
                }
            }

            if (_vms.Count > 0)
            {
                ComboVms.SelectedIndex = 0;
                ButtonSpice.IsEnabled = true;
            }
        }
        catch
        {
            _client = null;
            throw;
        }
    }

    private async void ButtonConnect_Click(object sender, RoutedEventArgs e)
    {
        await DoEvent(DoConnect);
    }

    private async Task DoSpice()
    {
        if (_client == null)
        {
            return;
        }
        if (ComboVms.SelectedItem is not VmWrapper vm)
        {
            return;
        }

        var (success, reasonPhrase, content) = await _client.Nodes[vm.Vm.Node].Qemu[vm.Vm.VmId].Spiceproxy.GetSpiceFileVVAsync(_client.Host);
        if (!success)
        {
            throw new Exception(reasonPhrase);
        }

        var tempFile = Path.ChangeExtension(Path.GetTempFileName(), ".vv");
        await File.WriteAllTextAsync(tempFile, content);
        var startInfo = new ProcessStartInfo(tempFile)
        {
            UseShellExecute = true
        };
        Process.Start(startInfo);
    }

    private async void ButtonSpice_Click(object sender, RoutedEventArgs e)
    {
        await DoEvent(DoSpice);
    }

    private async void MetroWindow_Closing(object? sender, CancelEventArgs e)
    {
        var dir = Path.GetDirectoryName(_settingsFile);
        if (!Directory.Exists(dir))
        {
            Directory.CreateDirectory(dir);
        }

        await File.WriteAllTextAsync(_settingsFile, JsonSerializer.Serialize(_settings));
    }

    private void ComboProfiles_OnSelectionChanged(object sender, RoutedEventArgs e)
    {
        _vms.Clear();
        _client = null;
        ButtonSpice.IsEnabled = false;
    }
}

internal class VmWrapper
{
    public VmWrapper(IClusterResourceVm vm)
    {
        Vm = vm;
    }

    public IClusterResourceVm Vm { get; set; }

    public override string ToString()
    {
        return $"{Vm.VmId} {Vm.Name}";
    }
}